@@ -36,7 +36,7 @@ constexpr unsigned API_VERSION_2_2 = ZIMG_MAKE_API_VERSION(2, 2);
 thread_local zimg_error_code_e g_last_error = ZIMG_ERROR_SUCCESS;
 thread_local std::string g_last_error_msg;
 
-constexpr unsigned VERSION_INFO[] = { 2, 9, 2 };
+constexpr unsigned VERSION_INFO[] = { 2, 9, 3 };
 
 
 template <class T, class U>
@@ -322,6 +322,8 @@ std::unique_ptr<zimg::resize::Filter> translate_resize_filter(zimg_resample_filt
 			return ztd::make_unique<zimg::resize::Spline16Filter>();
 		case ZIMG_RESIZE_SPLINE36:
 			return ztd::make_unique<zimg::resize::Spline36Filter>();
+		case ZIMG_RESIZE_SPLINE64:
+			return ztd::make_unique<zimg::resize::Spline64Filter>();
 		case ZIMG_RESIZE_LANCZOS:
 			param_a = std::isnan(param_a) ? zimg::resize::LanczosFilter::DEFAULT_TAPS : std::max(param_a, 1.0);
 			return ztd::make_unique<zimg::resize::LanczosFilter>(static_cast<unsigned>(param_a));
