@@ -4,6 +4,7 @@
 #include <cmath>
 #include <cstdint>
 #include <stdexcept>
+#include <type_traits>
 #include <vector>
 #include "common/align.h"
 #include "common/alloc.h"
@@ -326,7 +327,7 @@ protected:
 	{
 		auto attr = get_image_attributes();
 
-		ctx->cache_pos = 0;
+		ctx->cache_pos = attr.height;
 		ctx->source_left = attr.width;
 		ctx->source_right = 0;
 	}
@@ -364,7 +365,7 @@ public:
 
 	virtual void reset_context(ExecutionState *state) const = 0;
 
-	virtual void set_tile_region(ExecutionState *state, unsigned left, unsigned right, bool uv) const = 0;
+	virtual void set_tile_region(ExecutionState *state, unsigned top, unsigned left, unsigned right, bool uv) const = 0;
 
 	virtual void generate_until(ExecutionState *state, unsigned last, bool uv) const = 0;
 };
@@ -384,7 +385,7 @@ public:
 	size_t get_tmp_size(unsigned, unsigned) const override { return 0; }
 	void init_context(ExecutionState *, ExecutionStrategy) const override {}
 	void reset_context(ExecutionState *) const override {}
-	void set_tile_region(ExecutionState *, unsigned, unsigned, bool) const override {}
+	void set_tile_region(ExecutionState *, unsigned, unsigned, unsigned, bool) const override {}
 	void generate_until(ExecutionState *, unsigned, bool) const override {}
 };
 
@@ -417,7 +418,7 @@ public:
 
 	void request_external_cache(unsigned id) override
 	{
-		zassert_d(false, "attempt to set external cache on source node");
+		zassert_dfatal("attempt to set external cache on source node");
 	}
 
 	void complete() override {}
@@ -444,13 +445,14 @@ public:
 	void init_context(ExecutionState *state, ExecutionStrategy) const override { init_cache_context(state->get_node_state(get_id())); }
 	void reset_context(ExecutionState *state) const override { reset_cache_context(state->get_node_state(get_id())); }
 
-	void set_tile_region(ExecutionState *state, unsigned left, unsigned right, bool uv) const override
+	void set_tile_region(ExecutionState *state, unsigned top, unsigned left, unsigned right, bool uv) const override
 	{
 		auto *context = state->get_node_state(get_id());
 
 		left <<= uv ? m_subsample_w : 0;
 		right <<= uv ? m_subsample_w : 0;
 
+		context->cache_pos = std::min(context->cache_pos, top);
 		context->source_left = std::min(context->source_left, left);
 		context->source_right = std::max(context->source_right, right);
 	}
@@ -556,15 +558,17 @@ public:
 		return std::max(m_filter->get_tmp_size(left, right), m_parent->get_tmp_size(range.first, range.second));
 	}
 
-	void set_tile_region(ExecutionState *state, unsigned left, unsigned right, bool uv) const override
+	void set_tile_region(ExecutionState *state, unsigned top, unsigned left, unsigned right, bool uv) const override
 	{
 		auto *context = state->get_node_state(get_id());
 		auto range = m_filter->get_required_col_range(left, right);
 
+		context->cache_pos = std::min(context->cache_pos, top);
 		context->source_left = std::min(context->source_left, left);
 		context->source_right = std::max(context->source_right, right);
 
-		m_parent->set_tile_region(state, range.first, range.second, uv);
+		unsigned parent_top = m_filter->get_required_row_range(top).first;
+		m_parent->set_tile_region(state, parent_top, range.first, range.second, uv);
 	}
 };
 
@@ -616,10 +620,10 @@ public:
 		m_filter->init_context(state->get_context(get_id()), 0);
 	}
 
-	void set_tile_region(ExecutionState *state, unsigned left, unsigned right, bool uv) const override
+	void set_tile_region(ExecutionState *state, unsigned top, unsigned left, unsigned right, bool uv) const override
 	{
 		zassert_d(!uv, "request for chroma plane on luma node");
-		FilterNode::set_tile_region(state, left, right, false);
+		FilterNode::set_tile_region(state, top, left, right, false);
 	}
 
 	void generate_until(ExecutionState *state, unsigned last, bool uv) const override
@@ -707,10 +711,10 @@ public:
 		m_filter->init_context(static_cast<unsigned char *>(filter_ctx) + filter_ctx_size, 2);
 	}
 
-	void set_tile_region(ExecutionState *state, unsigned left, unsigned right, bool uv) const override
+	void set_tile_region(ExecutionState *state, unsigned top, unsigned left, unsigned right, bool uv) const override
 	{
 		zassert_d(uv, "request for luma plane on chroma node");
-		FilterNode::set_tile_region(state, left, right, true);
+		FilterNode::set_tile_region(state, top, left, right, true);
 	}
 
 	void generate_until(ExecutionState *state, unsigned last, bool uv) const override
@@ -829,16 +833,18 @@ public:
 		m_filter->init_context(state->get_context(get_id()), 0);
 	}
 
-	void set_tile_region(ExecutionState *state, unsigned left, unsigned right, bool) const override
+	void set_tile_region(ExecutionState *state, unsigned top, unsigned left, unsigned right, bool) const override
 	{
 		auto *context = state->get_node_state(get_id());
 		auto range = m_filter->get_required_col_range(left, right);
 
+		context->cache_pos = std::min(context->cache_pos, top);
 		context->source_left = std::min(context->source_left, left);
 		context->source_right = std::max(context->source_right, right);
 
-		m_parent->set_tile_region(state, range.first, range.second, false);
-		m_parent_uv->set_tile_region(state, range.first, range.second, true);
+		unsigned parent_top = m_filter->get_required_row_range(top).first;
+		m_parent->set_tile_region(state, parent_top, range.first, range.second, false);
+		m_parent_uv->set_tile_region(state, parent_top, range.first, range.second, true);
 	}
 
 	void generate_until(ExecutionState *state, unsigned last, bool uv) const override
@@ -851,13 +857,12 @@ public:
 		const ColorImageBuffer<void> &output_buffer = state->get_cache(get_cache_id())->buffer;
 
 		const ColorImageBuffer<const void> *real_input_buffer = &input_buffer;
-		ColorImageBuffer<const void> xbuffer;
+		std::aligned_storage<sizeof(ColorImageBuffer<const void>)>::type xbuffer;
 
 		if (m_parent->get_cache_id() != m_parent_uv->get_cache_id()) {
-			xbuffer[0] = input_buffer[0];
-			xbuffer[1] = input_buffer_uv[1];
-			xbuffer[2] = input_buffer_uv[2];
-			real_input_buffer = &xbuffer;
+			real_input_buffer = new (&xbuffer) ColorImageBuffer<const void>{
+				input_buffer[0], input_buffer_uv[1], input_buffer_uv[2]
+			};
 		}
 
 		for (; pos < last; pos += m_step) {
@@ -981,7 +986,7 @@ class FilterGraph::impl {
 
 		unsigned tile_width = static_cast<unsigned>(std::lrint(static_cast<double>(attr.width) * processor_cache / footprint));
 
-		if (tile_width > attr.width * 5 / 4)
+		if (tile_width > (attr.width / 5) * 4)
 			tile_width = attr.width;
 		else if (tile_width > attr.width / 2)
 			tile_width = ceil_n(attr.width / 2, ALIGNMENT);
@@ -1029,9 +1034,9 @@ class FilterGraph::impl {
 				node->reset_context(&state);
 			}
 
-			m_node->set_tile_region(&state, j, j_end, false);
+			m_node->set_tile_region(&state, 0, j, j_end, false);
 			if (m_node_uv)
-				m_node_uv->set_tile_region(&state, j >> m_subsample_w, j_end >> m_subsample_w, true);
+				m_node_uv->set_tile_region(&state, 0, j >> m_subsample_w, j_end >> m_subsample_w, true);
 
 			for (unsigned i = 0; i < attr.height; i += v_step) {
 				m_node->generate_until(&state, i + v_step, false);
@@ -1079,7 +1084,7 @@ class FilterGraph::impl {
 				node->reset_context(&state);
 			}
 
-			m_node->set_tile_region(&state, j, j_end, false);
+			m_node->set_tile_region(&state, 0, j, j_end, false);
 			m_node->generate_until(&state, attr.height, false);
 		}
 	}
@@ -1117,7 +1122,7 @@ class FilterGraph::impl {
 				node->reset_context(&state);
 			}
 
-			m_node_uv->set_tile_region(&state, j >> m_subsample_w, j_end >> m_subsample_w, true);
+			m_node_uv->set_tile_region(&state, 0, j >> m_subsample_w, j_end >> m_subsample_w, true);
 			m_node_uv->generate_until(&state, attr.height >> m_subsample_h, true);
 		}
 	}
@@ -1404,9 +1409,7 @@ void FilterGraph::callback::operator()(unsigned i, unsigned left, unsigned right
 	} catch (...) {
 		ret = 1;
 		// VS 15.7+ triggers an ICE here.
-#if !(_MSC_VER >= 1914 && _M_IX86)
-		zassert_d(false, "user callback must not throw");
-#endif
+		zassert_dfatal("user callback must not throw");
 	}
 
 	if (ret)
